package  fd.explicit;

import    rsf.vector.*;
import    jam.vector.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class AbsorbingBoundary {
  public static Rsf[] getVelocities(Rsf velocity) {
    switch(velocity.getDimension()) {
    case 2: 
      return AbsorbingBoundary.getVelocities2((Rsf2) velocity); 
    default:
      System.err.println("Error: not implemented yet");
    }
    return null;
  }
  static Rsf[] getVelocities2(Rsf2 velocity) {
    Rsf velocitySqr = (Rsf) velocity.getSpace().newMember();
    velocitySqr.addMul(velocity,velocity); 
    Rsf[] medium = new Rsf[6]; 
    for (int i = 0; i < medium.length; i++) {
      medium[i] = (Rsf) velocity.getSpace().newMember();
    }
    int ny = velocity.d   .length;
    int nx = velocity.d[0].length;
    float[][] m0 = ((Rsf2) medium[0]).d;
    float[][] m1 = ((Rsf2) medium[1]).d;
    float[][] m2 = ((Rsf2) medium[2]).d;
    float[][] m3 = ((Rsf2) medium[3]).d;
    float[][] m4 = ((Rsf2) medium[4]).d;
    float[][] m5 = ((Rsf2) medium[5]).d;

    float[][] vv = ((Rsf2) velocity).d;

    // inside weights
    medium[1].fill(2.f); medium[1].addScale(-4.f, velocitySqr);
    medium[0].copyFrom(velocitySqr);
    medium[2].copyFrom(velocitySqr);
    medium[3].copyFrom(velocitySqr);
    medium[4].copyFrom(velocitySqr);
    medium[5].fill(-1.f);

    // top side weights 
    for (int iy = 2; iy < ny-2; iy++) {
      m0[iy  ][0   ] = 0.f; 
      m1[iy  ][0   ] = 0.f; 
      m2[iy  ][0   ] = 0.f; 
      m3[iy  ][0   ] = 0.f; 
      m4[iy  ][0   ] = 0.f; 
      m5[iy  ][0   ] = 0.f; 
    }

    // bottom side weights 
    for (int iy = 2; iy < ny-2; iy++) {
      m1[iy  ][nx-1] = 1.f - vv[iy  ][nx-1]; 
      m0[iy  ][nx-1] = 0.f; 
      m2[iy  ][nx-1] = 0.f; 
      m3[iy  ][nx-1] =       vv[iy  ][nx-1]; 
      m4[iy  ][nx-1] = 0.f; 
      m5[iy  ][nx-1] = 0.f; 
    }

    // Left   side weights 
    for (int ix = 2; ix < nx-2; ix++) {
      m1[0   ][ix  ] = 1.f - vv[0   ][ix  ]; 
      m0[0   ][ix  ] = 0.f; 
      m2[0   ][ix  ] =       vv[0   ][ix  ]; 
      m3[0   ][ix  ] = 0.f; 
      m4[0   ][ix  ] = 0.f; 
      m5[0   ][ix  ] = 0.f; 
    }

    // Right   side weights 
    for (int ix = 2; ix < nx-2; ix++) {
      m1[ny-1][ix  ] = 1.f - vv[ny-1][ix  ]; 
      m0[ny-1][ix  ] =       vv[ny-1][ix  ]; 
      m2[ny-1][ix  ] = 0.f; 
      m3[ny-1][ix  ] = 0.f; 
      m4[ny-1][ix  ] = 0.f; 
      m5[ny-1][ix  ] = 0.f; 
    }

    // Right lower corner weights 
      m1[ny-1][nx-1] = 1.f - 2.f * vv[ny-1][nx-1];
      m1[ny-2][nx-1] = 1.f - 2.f * vv[ny-2][nx-1];
      m1[ny-1][nx-2] = 1.f - 2.f * vv[ny-1][nx-2];
      m0[ny-1][nx-1] =             vv[ny-1][nx-1]; 
      m0[ny-2][nx-1] =             vv[ny-2][nx-1]; 
      m0[ny-1][nx-2] =             vv[ny-1][nx-2]; 
      m2[ny-1][nx-1] = 0.f; 
      m2[ny-2][nx-1] = 0.f; 
      m2[ny-1][nx-2] = 0.f; 
      m3[ny-1][nx-1] =             vv[ny-1][nx-1]; 
      m3[ny-2][nx-1] =             vv[ny-2][nx-1]; 
      m3[ny-1][nx-2] =             vv[ny-1][nx-2]; 
      m4[ny-1][nx-1] = 0.f; 
      m4[ny-2][nx-1] = 0.f; 
      m4[ny-1][nx-2] = 0.f; 
      m5[ny-1][nx-1] = 0.f; 
      m5[ny-2][nx-1] = 0.f; 
      m5[ny-1][nx-2] = 0.f; 

    // Right upper corner weights 
      m1[ny-1][0] = 1.f - 2.f * vv[ny-1][0];
      m1[ny-2][0] = 1.f - 2.f * vv[ny-2][0];
      m1[ny-1][1] = 1.f - 2.f * vv[ny-1][1];
      m0[ny-1][0] =             vv[ny-1][0]; 
      m0[ny-2][0] =             vv[ny-2][0]; 
      m0[ny-1][1] =             vv[ny-1][1]; 
      m2[ny-1][0] = 0.f; 
      m2[ny-2][0] = 0.f; 
      m2[ny-1][1] = 0.f; 
      m3[ny-1][0] = 0.f; 
      m3[ny-2][0] = 0.f; 
      m3[ny-1][1] = 0.f; 
      m4[ny-1][0] =             vv[ny-1][0]; 
      m4[ny-2][0] =             vv[ny-2][0]; 
      m4[ny-1][1] =             vv[ny-1][1]; 
      m5[ny-1][0] = 0.f; 
      m5[ny-2][0] = 0.f; 
      m5[ny-1][1] = 0.f; 

    // Left upper corner weights 
      m1[0][0] = 1.f - 2.f * vv[0][0];
      m1[1][0] = 1.f - 2.f * vv[1][0];
      m1[0][1] = 1.f - 2.f * vv[0][1];
      m0[0][0] = 0.f; 
      m0[1][0] = 0.f; 
      m0[0][1] = 0.f; 
      m2[0][0] =             vv[0][0]; 
      m2[1][0] =             vv[1][0]; 
      m2[0][1] =             vv[0][1]; 
      m3[0][0] = 0.f; 
      m3[1][0] = 0.f; 
      m3[0][1] = 0.f; 
      m4[0][0] =             vv[0][0]; 
      m4[1][0] =             vv[1][0]; 
      m4[0][1] =             vv[0][1]; 
      m5[0][0] = 0.f; 
      m5[1][0] = 0.f; 
      m5[0][1] = 0.f; 

    // Left lower corner weights 
      m1[0][nx-1] = 1.f - 2.f * vv[0][nx-1];
      m1[1][nx-1] = 1.f - 2.f * vv[1][nx-1];
      m1[0][nx-2] = 1.f - 2.f * vv[0][nx-2];
      m0[0][nx-1] = 0.f; 
      m0[1][nx-1] = 0.f; 
      m0[0][nx-2] = 0.f; 
      m2[0][nx-1] =             vv[0][nx-1]; 
      m2[1][nx-1] =             vv[1][nx-1]; 
      m2[0][nx-2] =             vv[0][nx-2]; 
      m3[0][nx-1] =             vv[0][nx-1]; 
      m3[1][nx-1] =             vv[1][nx-1]; 
      m3[0][nx-2] =             vv[0][nx-2]; 
      m4[0][nx-1] = 0.f; 
      m4[1][nx-1] = 0.f; 
      m4[0][nx-2] = 0.f; 
      m5[0][nx-1] = 0.f; 
      m5[1][nx-1] = 0.f; 
      m5[0][nx-2] = 0.f; 

    return medium;
  } 
}



