package  fd.explicit;

import    sep.operator.SepLinearOperator;
import  juice.util.JuiceUtil;
import 	  rsf.vector.*;
import    jam.vector.*;

/**
 * This implements a finite difference approximation to one time step of 
 * the diffusion equation. 
 *
 * <BR><!--#include file="./source.inc"--><BR>
 * @author: Matt Schwab 
 */
public class DiffusionStepOp extends SepLinearOperator {
  /** material parameter: permeability */
  protected Rsf prm;

  public DiffusionStepOp(RsfSpace fieldSpc, Rsf prm) {
    super(fieldSpc);
    this.prm = prm;
  }

  protected void apply(boolean add, Vector wavein, Vector waveot) {

prm.write("prm.H");
    float[][] prm = ((Rsf2) this.prm).getData();
    int nx = ((Rsf) wavein).getAxis(0).nsamples; 
    int ny = ((Rsf) wavein).getAxis(1).nsamples; float[][] now;
    int nt = ((Rsf) wavein).getAxis(2).nsamples; float[][] neu;

    setOutputNull(isFwd, add, wavein, waveot);
    if (isFwd) for(int is=0; is<nt; is++) copySlice(wavein, waveot, is);
    else       for(int is=0; is<nt; is++) copySlice(waveot, wavein, is);

    if (isFwd) {
      now = ((Rsf3) waveot).getData()[0];
      neu = ((Rsf3) waveot).getData()[1];
      for (int y = 1; y < ny-1; y++){                         
      for (int x = 1; x < nx-1; x++){                         
	neu[y][x] += (1-4*prm[y][x])*  now[y  ][x  ] 
                        + prm[y][x] * (now[y-1][x  ] + now[y+1][x  ] 
                                     + now[y  ][x-1] + now[y  ][x+1]);
      }} 
      for(int it = 1; it < nt; it++) {
	now = ((Rsf3) waveot).getData()[it-1];
	neu = ((Rsf3) waveot).getData()[it  ]; 
	for (int y = 1; y < ny-1; y++){                         
	for (int x = 1; x < nx-1; x++){                         
	  neu[y][x] += (1-4*prm[y][x])*  now[y  ][x  ] 
                          + prm[y][x] * (now[y-1][x  ] + now[y+1][x  ] 
                                       + now[y  ][x-1] + now[y  ][x+1]); 
                        
	}}
      }
    }
    else {
      for(int it = nt-1; it > 0; it--) {
	now = ((Rsf3) wavein).getData()[it-1];
	neu = ((Rsf3) wavein).getData()[it  ]; 
	for (int y = 1; y < ny-1; y++){                         
        for (int x = 1; x < nx-1; x++){ 
	  now[y  ][x  ] += (1-4*prm[y][x])* neu[y][x];
	  now[y-1][x  ] +=      prm[y][x] * neu[y][x];
	  now[y+1][x  ] +=      prm[y][x] * neu[y][x];
	  now[y  ][x-1] +=      prm[y][x] * neu[y][x];
	  now[y  ][x+1] +=      prm[y][x] * neu[y][x];
	}}
      }
      now = ((Rsf3) wavein).getData()[0];
      neu = ((Rsf3) wavein).getData()[1];
      for (int y = 1; y < ny-1; y++){                         
      for (int x = 1; x < nx-1; x++){                         
	now[y  ][x  ] += (1-4*prm[y][x])* neu[y][x];
	now[y-1][x  ] +=      prm[y][x] * neu[y][x];
	now[y+1][x  ] +=      prm[y][x] * neu[y][x];
	now[y  ][x-1] +=      prm[y][x] * neu[y][x];
	now[y  ][x+1] +=      prm[y][x] * neu[y][x];
      }} 
    }
  }

  /** We need an extraction thing in Rsf */ 
  private void copySlice(Vector from, Vector to, int it) {
    float[][] ff =((Rsf3) from).d[it];
    float[][] tt =((Rsf3) to  ).d[it];
    for (int iy = 0; iy < tt   .length; iy++) 
    for (int ix = 0; ix < tt[0].length; ix++) 
      tt[iy][ix] = ff[iy][ix];
  }
  public boolean equals(Object obj2) {
    System.err.println("Warning: not fully implemented.");
    if (this == obj2)  return true;
    return false; 
  }
  public String toString() {
    return new String("DiffusionStep Operator \n permeability :\n" + prm);
  }
  public jam.JamObject copy() { 
    return (jam.JamObject) new DiffusionStepOp(this); 
  }
  protected DiffusionStepOp(DiffusionStepOp wavy) {
    super(wavy);
    System.err.println("copy constructor not implemented yet.");
  }
}     

