package  fd.explicit;

import    sep.operator.SepLinearOperator;
import  juice.util.JuiceUtil;
import 	  rsf.vector.*;
import    jam.vector.*;

/**
 * This implements a finite difference approximation to one time step of 
 * the wave equation. (forward inspired by Myzskowski's code example
 * (Scale, Theory of seismic imaging).
 *
 * <BR><!--#include file="./source.inc"--><BR>
 * @author: Matt Schwab 
 */
public class WaveStepOp extends SepLinearOperator {
  /** material parameter: velocity */
  protected Rsf vel;

  public WaveStepOp(RsfSpace fieldSpc, Rsf vel) {
    super(fieldSpc);
    this.vel = vel;
  }

  protected void apply(boolean add, Vector wavein, Vector waveot) {

((Rsf) wavein).write("in.H");
    float[][] vel = ((Rsf2) this.vel).getData();
    int nx = ((Rsf) wavein).getAxis(0).nsamples; float[][] old;
    int ny = ((Rsf) wavein).getAxis(1).nsamples; float[][] now;
    int nt = ((Rsf) wavein).getAxis(2).nsamples; float[][] neu;

    setOutputNull(isFwd, add, wavein, waveot);
    if (isFwd) for(int is=0; is<nt; is++) copySlice(wavein, waveot, is);
    else       for(int is=0; is<nt; is++) copySlice(waveot, wavein, is);

    if (isFwd) {
      now = ((Rsf3) waveot).getData()[0];
      neu = ((Rsf3) waveot).getData()[1];
      for (int y = 1; y < ny-1; y++){                         
      for (int x = 1; x < nx-1; x++){                         
	neu[y][x] += (2-4*vel[y][x])*  now[y  ][x  ] 
                        + vel[y][x] * (now[y-1][x  ] + now[y+1][x  ] 
                                     + now[y  ][x-1] + now[y  ][x+1]);
      }} 
      for(int it = 2; it < nt; it++) {
	old = ((Rsf3) waveot).getData()[it-2];
	now = ((Rsf3) waveot).getData()[it-1];
	neu = ((Rsf3) waveot).getData()[it  ]; 
	for (int y = 1; y < ny-1; y++){                         
	for (int x = 1; x < nx-1; x++){                         
	  neu[y][x] += (2-4*vel[y][x])*  now[y  ][x  ] 
                          + vel[y][x] * (now[y-1][x  ] + now[y+1][x  ] 
                                       + now[y  ][x-1] + now[y  ][x+1])
                                       - old[y  ][x  ];
	}}
      }
    }
    else {
      for(int it = nt-1; it > 1; it--) {
	old = ((Rsf3) wavein).getData()[it-2];
	now = ((Rsf3) wavein).getData()[it-1];
	neu = ((Rsf3) wavein).getData()[it  ]; 
	for (int y = 1; y < ny-1; y++){                         
        for (int x = 1; x < nx-1; x++){ 
	  now[y  ][x  ] += (2-4*vel[y][x])* neu[y][x];
	  now[y-1][x  ] +=      vel[y][x] * neu[y][x];
	  now[y+1][x  ] +=      vel[y][x] * neu[y][x];
	  now[y  ][x-1] +=      vel[y][x] * neu[y][x];
	  now[y  ][x+1] +=      vel[y][x] * neu[y][x];
	  old[y  ][x  ] +=                - neu[y][x];
	}}
      }
      now = ((Rsf3) wavein).getData()[0];
      neu = ((Rsf3) wavein).getData()[1];
      for (int y = 1; y < ny-1; y++){                         
      for (int x = 1; x < nx-1; x++){                         
	now[y  ][x  ] += (2-4*vel[y][x])* neu[y][x];
	now[y-1][x  ] +=      vel[y][x] * neu[y][x];
	now[y+1][x  ] +=      vel[y][x] * neu[y][x];
	now[y  ][x-1] +=      vel[y][x] * neu[y][x];
	now[y  ][x+1] +=      vel[y][x] * neu[y][x];
      }} 
    }
  }

  /** We need an extraction thing in Rsf */ 
  private void copySlice(Vector from, Vector to, int it) {
    float[][] ff =((Rsf3) from).d[it];
    float[][] tt =((Rsf3) to  ).d[it];
    for (int iy = 0; iy < tt   .length; iy++) 
    for (int ix = 0; ix < tt[0].length; ix++) 
      tt[iy][ix] = ff[iy][ix];
  }
  public boolean equals(Object obj2) {
    System.err.println("Warning: not fully implemented.");
    if (this == obj2)  return true;
    return false; 
  }
  public String toString() {
    return new String("WaveStep Operator \n velocity :\n" + vel);
  }
  public jam.JamObject copy() { return (jam.JamObject) new WaveStepOp(this); }
  protected WaveStepOp(WaveStepOp wavy) {
    super(wavy);
    System.err.println("copy constructor not implemented yet.");
  }
}     

