package  fd.frog;

import    sep.operator.SepLinearOperator;
import  juice.util.JuiceUtil;
import 	  rsf.vector.*;
import    jam.vector.*;

/**
 * This implements a finite difference approximation to the heat equation.
 *
 * <BR><!--#include file="./source.inc"--><BR>
 * @author: Matt Schwab (inspired by JFC's frog example)
 */
public class HeatOp extends SepLinearOperator {
  /** material parameter: heat conductivity divided by capacity */
  protected Rsf sigma;

  public HeatOp(Rsf sigma, boolean isFwd) {
    super(sigma.getSpace(), isFwd);
    if (sigma.getDimension()!= 2) 
      System.err.println("Err: Only x and time implemented yet.");
    this.sigma = sigma;
  }
  public HeatOp(Rsf sigma) {
    this(sigma, true);
  }
  /** copy constructor */
  protected HeatOp(HeatOp heater) {
    super(heater);
    System.err.println("copy constructor not implemented yet.");
  }

  protected void apply(boolean add, Vector source, Vector temper) {
    float[][] src = ((Rsf2) source).getData(); int nt = src.length;
    float[][] tmp = ((Rsf2) temper).getData(); int nx = src[1].length;
    float[][] sig = ((Rsf2) sigma ).getData();
    if (isFwd) temper.copyFrom(source); else source.copyFrom(temper);  // init
    if (isFwd) { 
      for (int it = 1; it < nt  ; it++){ int ip=it-1;             // step time
      for (int ix = 1; ix < nx-1; ix++){                          // step x 
	  tmp[it][ix] += tmp[ip][ix] + 
	    (tmp[ip][ix-1] -2.*tmp[ip][ix] + tmp[ip][ix+1]) * sig[it][ix];
      }}}
    else {  
      for (int it = nt-1; it > 0   ; it--){ int ip=it-1;          // step time
      for (int ix =    1; ix < nx-1; ix++){                       // step x 
	  src[ip][ix  ] +=   src[it][ix];
	  src[ip][ix-1] +=   src[it][ix] * sig[it][ix];
	  src[ip][ix  ] += - src[it][ix] * sig[it][ix] * 2;
	  src[ip][ix+1] +=   src[it][ix] * sig[it][ix];
      }}
    }
  }

  public boolean equals(Object obj2) {
    System.err.println("Warning: not fully implemented.");
    if (this == obj2)  return true;
    return false; 
  }
  public String toString() {
    return new String("Heat Operator \n Sigma :\n" + sigma);
  }
  public jam.JamObject copy() { return (jam.JamObject) new HeatOp(this); }
}




