package  fd.frog;

import    sep.operator.SepLinearOperator;
import  juice.util.JuiceUtil;
import 	  rsf.vector.*;
import    jam.vector.*;

/**
 * This implements a finite difference approximation to the equation of motion.
 *
 * <BR><!--#include file="./source.inc"--><BR>
 * @author: Matt Schwab (inspired by JFC's frog example)
 */
public class MotionOp extends SepLinearOperator {
  /** material parameter: incompressibility (could be sigma[x][y]*/
  protected Rsf density;

  public MotionOp(RsfSpace fieldGrid, Rsf density) {
    super(fieldGrid);
    this.density = density;
  }
  /** copy constructor */
  protected MotionOp(MotionOp motion) {
    super(motion);
    System.err.println("copy constructor not implemented yet.");
  }

  protected void apply(boolean add, Vector wavein, Vector waveot) {
    float[][][] in  = ((Rsf3) wavein).getData(); int ny = in[0]   .length;
    float[][][] ot  = ((Rsf3) waveot).getData(); int nx = in[0][0].length;
    float[][]   rho = ((Rsf2) density).getData(); 
    setOutputNull(isFwd, add, wavein, waveot);
    for (int y = 1; y < ny-1; y++){                          // step y 
    for (int x = 1; x < nx-1; x++){                          // step x 
      if (isFwd) { 
        ot[Wave.U][y][x]+=  in[Wave.U][y  ][x  ]
                          -(in[Wave.P][y  ][x+1] -in[Wave.P][y][x]) *rho[y][x];
	ot[Wave.V][y][x]+=  in[Wave.V][y  ][x  ] 
                          -(in[Wave.P][y+1][x  ] -in[Wave.P][y][x]) *rho[y][x];
        ot[Wave.P][y][x]+=  in[Wave.P][y  ][x  ];
      } else {
	in[Wave.P][y  ][x  ]+=   ot[Wave.P][y][x];
        in[Wave.U][y  ][x  ]+=   ot[Wave.U][y][x];
        in[Wave.P][y  ][x  ]+=   ot[Wave.U][y][x] *rho[y][x];
        in[Wave.P][y  ][x+1]+= - ot[Wave.U][y][x] *rho[y][x];
	in[Wave.V][y  ][x  ]+=   ot[Wave.V][y][x];
        in[Wave.P][y  ][x  ]+=   ot[Wave.V][y][x] *rho[y][x];
        in[Wave.P][y+1][x  ]+= - ot[Wave.V][y][x] *rho[y][x];
      }
    }}
  }

  public boolean equals(Object obj2) {
    System.err.println("Warning: not fully implemented.");
    if (this == obj2)  return true;
    return false; 
  }
  public String toString() {
    return new String("Motion Operator \n Incompressibility :\n" + density);
  }
  public jam.JamObject copy() { return (jam.JamObject) new MotionOp(this); }
}




