package  fd.frog;

import    sep.operator.SepLinearOperator;
import  juice.util.JuiceUtil;
import 	  rsf.vector.*;
import    jam.vector.*;

/**
 * This implements a finite difference approximation to the equation of State.
 *
 * <BR><!--#include file="./source.inc"--><BR>
 * @author: Matt Schwab (inspired by JFC's frog example)
 */
public class StateOp extends SepLinearOperator {
  /** material parameter: incompressibility (could be sigma[x][y]*/
  protected Rsf kappa;

  public StateOp(RsfSpace fieldGrid, Rsf kappa) {
    super(fieldGrid);
    this.kappa = kappa;
  }

  /** copy constructor */
  protected StateOp(StateOp state) {
    super(state);
    System.err.println("copy constructor not implemented yet.");
  }

  protected void apply(boolean add, Vector wavein, Vector waveot) {
    float[][][] in = ((Rsf3) wavein).getData(); int ny = in[0]   .length;
    float[][][] ot = ((Rsf3) waveot).getData(); int nx = in[0][0].length;
    float[][]   ka = ((Rsf2) kappa ).getData(); 
    setOutputNull(isFwd, add, wavein, waveot);
    for (int y = 1; y < ny-1; y++){                          // step y 
    for (int x = 1; x < nx-1; x++){                          // step x 
      if (isFwd) { 
        ot[Wave.P][y][x] += in[Wave.P][y][x] 
	                 - (in[Wave.U][y][x]-in[Wave.U][y  ][x-1])*ka[y][x] 
	                 - (in[Wave.V][y][x]-in[Wave.V][y-1][x  ])*ka[y][x];
        ot[Wave.U][y][x] += in[Wave.U][y][x];
        ot[Wave.V][y][x] += in[Wave.V][y][x];
      } 
      else {
        in[Wave.P][y  ][x  ]+=  ot[Wave.P][y][x];
        in[Wave.U][y  ][x  ]+= -ot[Wave.P][y][x]*ka[y][x]+ot[Wave.U][y][x];
        in[Wave.U][y  ][x-1]+=  ot[Wave.P][y][x]*ka[y][x];
        in[Wave.V][y  ][x  ]+= -ot[Wave.P][y][x]*ka[y][x]+ot[Wave.V][y][x];
        in[Wave.V][y-1][x  ]+=  ot[Wave.P][y][x]*ka[y][x];
      }
    }}
  }

  public boolean equals(Object obj2) {
    System.err.println("Warning: not fully implemented.");
    if (this == obj2)  return true;
    return false; 
  }
  public String toString() {
    return new String("State Operator \n Incompressibility :\n" + kappa);
  }
  public jam.JamObject copy() { return (jam.JamObject) new StateOp(this); }
}




