package  fd.frog;

import    sep.operator.SepLinearOperator;
import  juice.util.JuiceUtil;
import 	  rsf.vector.*;
import    jam.vector.*;

/**
 * This implements a finite difference approximation to the heat equation.
 *
 * <BR><!--#include file="./source.inc"--><BR>
 * @author: Matt Schwab (inspired by JFC's frog example)
 */
public class ViscosityOp extends SepLinearOperator {
  /** material parameter: viscosity (could be sigma[x][y]*/
  protected Rsf sigma;

  public ViscosityOp(RsfSpace fieldGrid, Rsf sigma) {
    super(fieldGrid);
    this.sigma = sigma;
  }
  /** copy constructor */
  protected ViscosityOp(ViscosityOp visco) {
    super(visco);
    System.err.println("copy constructor not implemented yet.");
  }

  protected void apply(boolean add, Vector wavein, Vector waveot) {
    float[][][] in = ((Rsf3) wavein).getData(); int ny = in[0]   .length;
    float[][][] ot = ((Rsf3) waveot).getData(); int nx = in[0][0].length;
    float[][]   sg = ((Rsf2) sigma ).getData();
    //    setOutputNull(isFwd, add, wavein, waveot);         // viscosity 
    for (int y = 1; y < ny-1; y++){                          // step y 
    for (int x = 1; x < nx-1; x++){                          // step x 
      if (isFwd) { 
	ot[Wave.P][y  ][x  ] +=  in[Wave.P][y][x] +
       (in[Wave.P][y  ][x-1]-2*in[Wave.P][y][x]+in[Wave.P][y  ][x+1])*sg[y][x]+
       (in[Wave.P][y-1][x  ]-2*in[Wave.P][y][x]+in[Wave.P][y+1][x  ])*sg[y][x];
        ot[Wave.U][y  ][x  ] += in[Wave.U][y][x];
        ot[Wave.V][y  ][x  ] += in[Wave.V][y][x];
      } 
      else {
	in[Wave.U][y  ][x  ] += ot[Wave.U][y][x];
	in[Wave.V][y  ][x  ] += ot[Wave.V][y][x];
	in[Wave.P][y  ][x  ] += ot[Wave.P][y][x] * ( 1 - sg[y][x] * 4);
	in[Wave.P][y  ][x-1] += ot[Wave.P][y][x] *       sg[y][x];
	in[Wave.P][y  ][x+1] += ot[Wave.P][y][x] *       sg[y][x];
	in[Wave.P][y-1][x  ] += ot[Wave.P][y][x] *       sg[y][x];
	in[Wave.P][y+1][x  ] += ot[Wave.P][y][x] *       sg[y][x];
      }
    }}
  }

  public boolean equals(Object obj2) {
    System.err.println("Warning: not fully implemented.");
    if (this == obj2)  return true;
    return false; 
  }
  public String toString() {
    return new String("Viscosity Operator \n Sigma :\n" + sigma);
  }
  public jam.JamObject copy() { return (jam.JamObject) new ViscosityOp(this); }
}




