package  fd.frog;

import    sep.operator.SepLinearOperator;
import  juice.operator.*;
import 	  rsf.vector.*;
import    jam.vector.*;
import    jam.operator.*;

/**
 * This implements a finite difference approximation to the equation of Wave.
 *
 * <BR><!--#include file="./source.inc"--><BR>
 * @author: Matt Schwab (inspired by JFC's frog example)
 */
public class WaveOp extends SepLinearOperator {
  private Wave grid;
  private Rsf  incompressibility;
  private Rsf  density;
  private Rsf  viscosity; 

  public WaveOp(Wave grid, Rsf incompressibility, Rsf density, Rsf viscosity) {
    super(grid.getGrid());
    this.grid              = grid;
    this.incompressibility = incompressibility;
    this.density           = density;
    this.viscosity         = viscosity; 
  }
  /** copy constructor */
  protected WaveOp(WaveOp wave) {
    super(wave);
    System.err.println("copy constructor not implemented yet.");
  }

  protected void apply(boolean add, Vector wavein, Vector waveot) {
    Rsf wavett = (Rsf) grid.getFieldGrid().newMember();
    Rsf wavess = (Rsf) grid.getFieldGrid().newMember();
    int nt     =((Rsf) wavein).getAxis(3).nsamples;

    hasAdjoint  state = new     StateOp(grid.getFieldGrid(),incompressibility);
    hasAdjoint motion = new    MotionOp(grid.getFieldGrid(),density);
    hasAdjoint viscos = new ViscosityOp(grid.getFieldGrid(),viscosity);
    CompoundLinOpAdj wave = new CompoundLinOpAdj(state,motion,viscos);

    setOutputNull(isFwd, add, wavein, waveot);
    if (isFwd) { 
        copySlice( wavein, waveot,  0);
        copySlice( wavein, wavett,  0);
      for (int it = 1; it < nt; it++) {
	copySlice( wavein, wavess, it); // load source 
	wave.addImage(wavett, wavess    ); // combine source and last time step
	copySlice( wavess, wavett, it); // feedback last time step
	copySlice( wavess, waveot, it); // extract solution
      }
    }
    else {
        copySlice( waveot, wavein, nt-1);
        copySlice( waveot, wavess, nt-1);
      for (int it = nt-2; it > -1; it--) {
	copySlice( waveot, wavett, it  );
	wave.adjoint().addImage(wavess, wavett); 
	copySlice( wavett, wavess, it  );
	copySlice( wavett, wavein, it  );
      }
    } 
  }

  /** We need an extraction thing in Rsf */ 
  private void copySlice(Vector from, Vector to, int it) {
    Rsf ffRsf = (Rsf) from; float[][][] ff;
    Rsf ttRsf = (Rsf) to  ; float[][][] tt;
    if (ffRsf.getDimension() == 4) ff =((Rsf4)ffRsf).d[it];
    else                           ff =((Rsf3)ffRsf).d;
    if (ttRsf.getDimension() == 4) tt =((Rsf4)ttRsf).d[it];
    else                           tt =((Rsf3)ttRsf).d;
    for (int ii = 0; ii < tt      .length; ii++) 
    for (int iy = 0; iy < tt[0]   .length; iy++) 
    for (int ix = 0; ix < tt[0][0].length; ix++) 
      tt[ii][iy][ix] = ff[ii][iy][ix];
  }

  public boolean equals(Object obj2) {
    System.err.println("Warning: not fully implemented.");
    if (this == obj2)  return true;
    return false; 
  }
  public String toString() {
    return new String("Wave Operator "                               +  
                      "\n Incompressibility :\n" + incompressibility + 
		      "\n Density :          \n" + density           + 
		      "\n Viscosity :        \n" + viscosity);
  }
  public jam.JamObject copy() { return (jam.JamObject) new WaveOp(this); }
}




